SystemOrganization addCategory: #MECCG!
SystemOrganization addCategory: 'MECCG-Model'!
SystemOrganization addCategory: 'MECCG-Model-Tests'!
SystemOrganization addCategory: 'MECCG-Support'!

TestCase subclass: #ModelUsageExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!ModelUsageExamples methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 12:01'!
checkLogger: logger toMakeSureThePlayerOrderIsTheSameAsIn: players 
	| logMessages |
	logMessages  := players collect: [ :player | 
		'Begin turn of player ', player name ].
	self assert: logger logSize >= players size.
	self assert: (logger areTheseMessagesOrdered: logMessages).! !

!ModelUsageExamples methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 09:27'!
test_01_SimpleUsage
	"Modelar empezar un juego con N jugadores que empieza (Game Setup), 
	se juegan turnos y termina (Victory Condition)"
	| match result logger |

	match := Match of: MiddleEarthCardGame rules.
	logger := MECCGTranscriptLogger on: match.
	logger logEvents: #somethingHappened.
	logger logOn: Transcript.
	match addPlayer: (PlayerOfMECCG named: 'Fede Padawan').
	match addPlayer: (PlayerOfMECCG named: 'Pieter').
	self assert: match result isNil.
	match setup.
	self assert: match result notNil.
	self assert: match isReady.
	self assert: match result matchDuration isNil.
	match play.
	match wrapUp.
	result := match result.
	self assert: result matchDuration notNil.
	logger detach.! !

!ModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 17:28'!
test_02_PlayerOrder
	"We test whether the player who's supposed to go first, actually does."
	
	| match players logger |
	match := Match of: MiddleEarthCardGame rules.
	logger := MECCGLogger on: match.
	players := OrderedCollection
		with: (PlayerOfMECCG named: 'Federico')
		with: (PlayerOfMECCG named: 'Sergio')
		with: (PlayerOfMECCG named: 'Emanuel')
		with: (PlayerOfMECCG named: 'Pablo').
	
	logger logEvents: #somethingHappened.
	match addPlayers: players.
	match setup.
	self assert: match isReady.
	match play.
	match wrapUp.
	logger detach.
	self 
		checkLogger: logger 
		toMakeSureThePlayerOrderIsTheSameAsIn: players.! !

!ModelUsageExamples methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 09:27'!
test_03_LastActionOfThePhase
	"We test that, for each phase, the last action taken is the 'EndPhase' action.
	We also test that the phases of the turn occur in the right order."

	^ self assert: false.! !

!ModelUsageExamples methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 09:27'!
test_04_MatchProgress
	"We test the match progresses in the expected way."

	^ self assert: false.! !

Object subclass: #Action
	instanceVariableNames: 'actor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Action methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:07'!
actor
	^ actor! !

!Action methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:07'!
actor: anObject
	actor := anObject! !

!Action methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:06'!
perform
! !

Action subclass: #EndPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

Object subclass: #MECCGListener
	instanceVariableNames: 'source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!MECCGListener commentStamp: 'SergioFedi 4/29/2016 09:46' prior: 0!
I am a Listener (as in the Observer pattern) of MECCG Games, Matches, Players, etc.
!

!MECCGListener class methodsFor: 'instance creation' stamp: 'SergioFedi 4/29/2016 09:55'!
on: anObject

	^self new source: anObject.! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:49'!
detach

	self source removeActionsWithReceiver: self.! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:45'!
logEventOccurence: aString
	self subclassResponsibility! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:49'!
logEvents: aSymbol

	self source 
		when: aSymbol
		send: #logEventOccurence:
		to: self.! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:49'!
source
	^ source! !

!MECCGListener methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:49'!
source: anObject
	source := anObject! !

MECCGListener subclass: #MECCGLogger
	instanceVariableNames: 'log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!MECCGLogger commentStamp: 'SergioFedi 4/29/2016 09:51' prior: 0!
I listen to events of a MECCG and log them in a collection.!

!MECCGLogger methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 12:13'!
areTheseMessagesOrdered: aSequenceableCollection

	^self log doTheseObjectsAppearInOrder: aSequenceableCollection.! !

!MECCGLogger methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 11:10'!
initialize

	self log: StringOfObjects new.! !

!MECCGLogger methodsFor: 'accessing' stamp: 'SergioFedi 4/29/2016 09:53'!
log
	^ log! !

!MECCGLogger methodsFor: 'accessing' stamp: 'SergioFedi 4/29/2016 09:53'!
log: anObject
	log := anObject! !

!MECCGLogger methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:51'!
logEventOccurence: aString

	self log add: aString.! !

!MECCGLogger methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 12:09'!
logSize

	^self log size.! !

MECCGListener subclass: #MECCGTranscriptLogger
	instanceVariableNames: 'sink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!MECCGTranscriptLogger commentStamp: 'SergioFedi 4/28/2016 16:58' prior: 0!
This class listens other objects and logs the events that are of interest.!

!MECCGTranscriptLogger methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:51'!
logEventOccurence: aString

	self sink nextPutAll: aString.
	self sink cr.
	self sink flush.! !

!MECCGTranscriptLogger methodsFor: 'public' stamp: 'SergioFedi 4/28/2016 19:55'!
logOn: aThreadSafeTranscript

	self sink: aThreadSafeTranscript.! !

!MECCGTranscriptLogger methodsFor: 'private' stamp: 'SergioFedi 4/28/2016 19:48'!
sink
	^ sink! !

!MECCGTranscriptLogger methodsFor: 'private' stamp: 'SergioFedi 4/28/2016 19:48'!
sink: anObject
	sink := anObject! !

Object subclass: #MECCGPhase
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MECCGPhase commentStamp: 'SergioFedi 4/26/2016 08:56' prior: 0!
I model a phase of a turn of the Middle Earth Collectible Card Game.
!

!MECCGPhase class methodsFor: 'instance creation' stamp: 'SergioFedi 4/26/2016 08:57'!
named: aString

	^self new name: aString.! !

!MECCGPhase methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:09'!
availableActionsForPlayer: aPlayerOfMECCG inMatch: aMatch

	^Array
		with: Action new 
		with: EndPhase new.! !

!MECCGPhase methodsFor: 'accessing' stamp: 'SergioFedi 4/26/2016 08:56'!
name
	^ name! !

!MECCGPhase methodsFor: 'accessing' stamp: 'SergioFedi 4/26/2016 08:56'!
name: anObject
	name := anObject! !

Object subclass: #Match
	instanceVariableNames: 'rules players result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Match commentStamp: 'SergioFedi 4/21/2016 08:22' prior: 0!
I model a match of a game.

You can see how to use me in ModelUsageExamples
!

!Match class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:12'!
of: matchRules

	^self new
		rules: matchRules.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 09:02'!
aboutToBeginTurnOf: aPlayerOfMECCG 
	self triggerEvent: #somethingHappened with: 'Begin turn of player ', aPlayerOfMECCG name.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 08:30'!
addPlayer: aPlayer

	aPlayer match: self.
	players add: aPlayer.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 08:44'!
addPlayers: aCollectionOfPlayers 
	aCollectionOfPlayers do: [ :player | 
		self addPlayer: player].! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:19'!
initialize
	super initialize.
	players := OrderedCollection new.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:33'!
isReady

	^self rules isMatchReady: self.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/28/2016 22:38'!
play

	self triggerEvent: #somethingHappened with: 'Match started'.
	self rules play: self.
! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:48'!
players
	^ players! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:40'!
result

	^result.! !

!Match methodsFor: 'accesing' stamp: 'SergioFedi 4/17/2016 17:49'!
result: aMatchResult

	result := aMatchResult.! !

!Match methodsFor: 'accessing' stamp: 'SergioFedi 4/17/2016 17:13'!
rules
	^ rules! !

!Match methodsFor: 'accessing' stamp: 'SergioFedi 4/17/2016 17:13'!
rules: anObject
	rules := anObject! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/28/2016 22:35'!
setup

	(self rules isMatchReadyForSetup: self)
		ifTrue: [
			self rules setupMatch: self ].! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:57'!
wrapUp

	self rules wrapUpMatch: self.! !

Object subclass: #MatchOutcome
	instanceVariableNames: 'matchStatistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MatchOutcome commentStamp: 'SergioFedi 4/21/2016 08:27' prior: 0!
I model the outcome of a match.

The particular data I will contain will be determined by the rules of the match.
!

!MatchOutcome methodsFor: 'as yet unclassified' stamp: 'SergioFedi 4/22/2016 08:59'!
initialize
	super initialize.
	matchStatistics := Dictionary new.
	self matchDuration: nil.
	self isFinished: false.! !

!MatchOutcome methodsFor: 'accesing' stamp: 'SergioFedi 4/22/2016 08:58'!
isFinished

	^matchStatistics at: 'isFinished'.! !

!MatchOutcome methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:58'!
isFinished: anObject 

	matchStatistics at: 'isFinished' put: anObject.! !

!MatchOutcome methodsFor: 'accesing' stamp: 'SergioFedi 4/17/2016 17:50'!
matchDuration

	^matchStatistics at: 'matchDuration'.! !

!MatchOutcome methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:56'!
matchDuration: anObject 

	matchStatistics at: 'matchDuration' put: anObject.! !

Object subclass: #MiddleEarthCardGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MiddleEarthCardGame commentStamp: 'SergioFedi 4/21/2016 08:31' prior: 0!
I model the game of Middle Earth Collectible Card Game.
I can answer the rules of the game.
!

!MiddleEarthCardGame class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:09'!
rules

	^MiddleEarthCardGameRules new.! !

Object subclass: #MiddleEarthCardGameRules
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MiddleEarthCardGameRules commentStamp: 'SergioFedi 4/21/2016 08:36' prior: 0!
I model the rules of the Middle Earth Card Game.

My responsibility is to run matches of that game.
!

!MiddleEarthCardGameRules methodsFor: 'private' stamp: 'SergioFedi 4/22/2016 09:03'!
incrementRoundCountOf: aMatch

	aMatch result matchDuration: aMatch result matchDuration + 1.! !

!MiddleEarthCardGameRules methodsFor: 'private' stamp: 'SergioFedi 4/22/2016 09:05'!
initializeRoundCountOf: aMatch 

	aMatch result matchDuration: 0.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 09:01'!
isFinished: aMatch

	^aMatch result isFinished.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:34'!
isMatchReady: aMatch

	^true.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:31'!
isMatchReadyForSetup: aMatch

	^true.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 12:26'!
play: aMatch
	"Play the match until it is finished"

	self initializeRoundCountOf: aMatch.
	[self isFinished: aMatch] whileFalse: [
			self playRoundOf: aMatch.
			self incrementRoundCountOf: aMatch.
	].
! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 08:44'!
playRoundOf: aMatch
	aMatch players
		do: [ :player |
			self playTurnOf: player in: aMatch.
			(self isFinished: aMatch)
				ifTrue: [ ^ self ] ]! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 09:20'!
playTurnOf: aPlayer in: aMatch

	aMatch aboutToBeginTurnOf: aPlayer.
	self turnPhases do: [ :phase |
		aPlayer playPhase: phase.
	].

	"
	self turnPhases do: [ :phase |
		| chosenAction |
		chosenAction := aPlayer actionToPlayInPhase: phase.
		self performAction: chosenAction.
	].
	"

	(aMatch result matchDuration >= 3) ifTrue: [ 
		aMatch result isFinished: true.
	]! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/21/2016 08:28'!
setupMatch: aMatch

	aMatch result: MatchOutcome new.
	^self.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 08:59'!
turnPhases

	^Array
		with: (MECCGPhase named: 'Untap')
		with: (MECCGPhase named: 'Organization')
		with: (MECCGPhase named: 'Long Event')
		with: (MECCGPhase named: 'Movement / Hazard')
		with: (MECCGPhase named: 'Site')
		with: (MECCGPhase named: 'End of Turn').
		! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:57'!
wrapUpMatch: aMatch

	^self.! !

Object subclass: #Player
	instanceVariableNames: 'name match'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Player commentStamp: 'SergioFedi 4/21/2016 08:30' prior: 0!
I model a participant of a Match that plays in it.
!

!Player class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:16'!
named: aString

	^self new name: aString.! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 08:30'!
match
	^ match! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 08:30'!
match: anObject
	match := anObject! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:15'!
name
	^ name! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:15'!
name: anObject
	name := anObject! !

Player subclass: #PlayerOfMECCG
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!PlayerOfMECCG commentStamp: 'SergioFedi 4/25/2016 08:36' prior: 0!
I model a participant of a Match of MiddleEarthCardGame that plays in it.!

!PlayerOfMECCG methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:09'!
chooseActionAmong: aCollection

	^aCollection atRandom.! !

!PlayerOfMECCG methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:01'!
playPhase: aPhase

	| availableActions actionToDo |
	availableActions := aPhase availableActionsForPlayer: self inMatch: self match.
	actionToDo := self chooseActionAmong: availableActions.
	^actionToDo! !

Object subclass: #StringOfObjects
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!StringOfObjects commentStamp: 'SergioFedi 5/2/2016 11:10' prior: 0!
I generalize the concept of String.

From String of Characters to a String of Objects.

For the moment my purpose is to solve the pattern matching problem for the more general case of matching a pattern of objects instead of a pattern of characters.
!

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 11:26'!
add: anObject
 
	objects add: anObject.! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 13:51'!
alphabetFor: aCollection

	^(1 to: aCollection size) collect: [ :n | Character value: 64 + n ].! !

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 17:04'!
doTheseObjectsAppearInOrder: aCollection

	| areOrdered logString regularExpresion |
	logString := self logAsStringFor: aCollection.
	regularExpresion := self objectsAppearInOrderRegExpFor: aCollection.
	areOrdered := logString matchesRegex: regularExpresion.
	^areOrdered.! !

!StringOfObjects methodsFor: 'as yet unclassified' stamp: 'SergioFedi 5/2/2016 11:26'!
initialize

	super initialize.
	objects := OrderedCollection new.! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 17:20'!
logAsStringFor: aCollection

	| map string |
	string := WriteStream on: String new.
	map := Dictionary new.
	(self alphabetFor: aCollection) with: aCollection do: [ :character :object |
		map at: object put: character.
	].
	objects do: [ :object |
		string nextPut: (map at: object ifAbsent: [$_])
	].
	^string contents.
	! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 19:02'!
objectsAppearInOrderRegExpFor: aCollection

	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: '[^ABCD]*', '(A(B(CD?)?)?)?', '\w*'.
"	stream nextPutAll: '[^ABCD]*', '((A(B(CD?)?)?)?)+', '\w*'."
	^stream contents.
	
"	'_ABCDABCDABCDA___' matchesRegex: '\w*ABCD\w*'.
	'_ABDC___' matchesRegex: '_*', '(A(B(CD?)?)?)?', '\w*'.
	'_12fth___' matchesRegex: '[^ABCD]*', 'A?'.
"
! !

!StringOfObjects methodsFor: 'as yet unclassified' stamp: 'SergioFedi 5/2/2016 11:36'!
size

	^objects size.! !
